package jackson.exceptions;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;

public class ExceptionsRunner {

	public static void main(String[] args) {
		jacksonNoConstructor();
		jacksonListToObject();
		privateFieldAndGetter();
		stringToIntegerIncomaptible();
		unrecogonizedProperty();
		wrongJsonRoot();
		code39Exception();
	}

	private static void jacksonNoConstructor() {
		ObjectMapper mapper = new ObjectMapper();
		String json = "{\"id\":1,\"name\":\"John\"}";
		try {
			mapper.readValue(json, Shop.class);
		} catch (JsonProcessingException e) {
			System.out.println(e.getClass().getName() + " : " + e.getOriginalMessage());
			// e.printStackTrace();
		}
	}

	private static void jacksonListToObject() {
		ObjectMapper mapper = new ObjectMapper();
		String json = "[{\"id\":1,\"name\":\"John\"},{\"id\":2,\"name\":\"Adam\"}]";
		try {
			mapper.readValue(json, Customer.class);
		} catch (JsonProcessingException e) {
			System.out.println(e.getClass().getName() + " : " + e.getOriginalMessage());
			// e.printStackTrace();
		}
	}

	private static void privateFieldAndGetter() {
		ObjectMapper mapper = new ObjectMapper();
		try {
			Product p = new Product(1, "Anmol");
			mapper.writeValueAsString(p);
		} catch (JsonProcessingException e) {
			System.out.println(e.getClass().getName() + " : " + e.getOriginalMessage());
			// e.printStackTrace();
		}
	}

	private static void stringToIntegerIncomaptible() {
		ObjectMapper mapper = new ObjectMapper();
		String json = "{\"id\":\"Three\",\"name\":\"Anmol\"}";
		try {
			mapper.readValue(json, Customer.class);
		} catch (JsonProcessingException e) {
			System.out.println(e.getClass().getName() + " : " + e.getOriginalMessage());
			// e.printStackTrace();
		}
	}

	private static void unrecogonizedProperty() {
		ObjectMapper mapper = new ObjectMapper();
		String json = "{\"id\":99,\"name\":\"Anmol\",\"location\":\"Bangalore\"}";
		try {
			mapper.readValue(json, Customer.class);
		} catch (JsonProcessingException e) {
			System.out.println(e.getClass().getName() + " : " + e.getOriginalMessage());
			// e.printStackTrace();
		}
	}

	private static void wrongJsonRoot() {
		ObjectMapper mapper = new ObjectMapper();
		String json = "{\"jsonRoot\" : {\"id\":1,\"name\":\"John\"}}";
		try {
			mapper.enable(DeserializationFeature.UNWRAP_ROOT_VALUE);
			mapper.readValue(json, Customer.class);
		} catch (JsonProcessingException e) {
			System.out.println(e.getClass().getName() + " : " + e.getOriginalMessage());
			// e.printStackTrace();
		}
	}

	private static void code39Exception() {
		ObjectMapper mapper = new ObjectMapper();
		String json = "{'id':99,'name':'Anmol'}";
		try {
			mapper.readValue(json, Employee.class);
		} catch (JsonProcessingException e) {
			System.out.println(e.getClass().getName() + " : " + e.getOriginalMessage());
			// e.printStackTrace();
		}
	}
}
