package jackson.solutions;

import java.util.List;

import com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility;
import com.fasterxml.jackson.annotation.PropertyAccessor;
import com.fasterxml.jackson.core.JsonFactory;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;

import jackson.exceptions.Customer;
//import jackson.exceptions.Customer;
import jackson.exceptions.Employee;
import jackson.exceptions.Product;

public class Runner {
	public static void main(String[] args) {
		jacksonListToObjectFix();
		privateFieldAndGetterFix();
		unrecogonizedPropertyFix();
		wrongJsonRootFix();
		code39Fix();
	}

	private static void jacksonListToObjectFix() {
		ObjectMapper mapper = new ObjectMapper();
		String json = "[{\"id\":1,\"name\":\"John\"}," + "{\"id\":2,\"name\":\"Adam\"}]";
		try {
			List<Customer> customer = mapper.readValue(json, new TypeReference<List<Customer>>() {
			});
			System.out.println("jacksonListToObjectFix " + customer);
			// Prints [Customer [id=1, name=John],
			// Customer [id=2, name=Adam]]
		} catch (JsonProcessingException e) {
			System.out.println(e.getClass().getName() + " : " + e.getOriginalMessage());
		}
	}

	private static void privateFieldAndGetterFix() {
		ObjectMapper mapper = new ObjectMapper();
		try {
			Product p = new Product(1, "Anmol");
			mapper.setVisibility(PropertyAccessor.FIELD, Visibility.ANY);
			String json = mapper.writeValueAsString(p);
			System.out.println("privateFieldAndGetterFix - Solution 1" + json);
			// Prints {"id":1,"name":"Anmol"}

			// Solution 2
			Product_Class_Fix p1 = new Product_Class_Fix(1, "Anmol");
			ObjectMapper mapper2 = new ObjectMapper();
			json = mapper2.writeValueAsString(p1);
			System.out.println("privateFieldAndGetterFix - Solution 2" + json);

			// Solution 3
			Product_Class_Fix2 p2 = new Product_Class_Fix2(1, "Anmol");
			ObjectMapper mapper3 = new ObjectMapper();
			json = mapper3.writeValueAsString(p2);
			System.out.println("privateFieldAndGetterFix - Solution 3" + json);
			// Prints {"id":1,"name":"Anmol"}

		} catch (JsonProcessingException e) {
			System.out.println(e.getClass().getName() + " : " + e.getOriginalMessage());
		}
	}

	private static void code39Fix() {
		String json = "{'id':99,'name':'Anmol'}";
		try {
			JsonFactory jf = new JsonFactory();
			jf.enable(JsonParser.Feature.ALLOW_SINGLE_QUOTES);
			ObjectMapper mapper = new ObjectMapper(jf);
			Employee employee = mapper.readValue(json, Employee.class);
			System.out.println("code39Fix " + employee.id + " = " + employee.name);
			// Prints 99 = Anmol
		} catch (JsonProcessingException e) {
			System.out.println(e.getClass().getName() + " : " + e.getOriginalMessage());
		}
	}

	private static void wrongJsonRootFix() {
		ObjectMapper mapper = new ObjectMapper();
		String json = "{\"jsonRoot\" : {\"id\":1,\"name\":\"John\"}}";
		try {
			mapper.enable(DeserializationFeature.UNWRAP_ROOT_VALUE);
			Customer_Class_Fix customerClassFix = mapper.readValue(json, Customer_Class_Fix.class);
			System.out.println("wrongJsonRootFix " + customerClassFix.name + " - " + customerClassFix.id);
			// Print John - 1
		} catch (JsonProcessingException e) {
			System.out.println(e.getClass().getName() + " : " + e.getOriginalMessage());
		}
	}

	private static void unrecogonizedPropertyFix() {
		ObjectMapper mapper = new ObjectMapper();
		String json = "{\"id\":99,\"name\":\"Anmol\",\"location\":\"Bangalore\"}";
		mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
		try {
			Customer customer = mapper.readValue(json, Customer.class);
			System.out.println("unrecogonizedPropertyFix " + customer);

			// solution 2
			Customer_Class_Fix2 customerClassFix2 = mapper.readValue(json, Customer_Class_Fix2.class);
			System.out.println("unrecogonizedPropertyFix" + customerClassFix2);
		} catch (JsonProcessingException e) {
			System.out.println(e.getClass().getName() + " : " + e.getOriginalMessage());
		}
	}
}
